/*************************************************************
*     pwm.c  PWM Data Write  Ver.1.0                         *
*                                          Date:14 May 2004  *
*                                Programed by Hiroki.Segawa  *
*************************************************************/

/*************************************************************************
*       This Source file contains these function.                        *
*                                                                        *
* void pwm_both_data(int,int);       PWM DUTY Mode function              *
* void pwm_straight(int,int);        PWM PID Mode(straight) function     *
* void pwm_rotate(int);              PWM PID Mode(rotate) function       *
* void pwm_lqr_straight(int,int);    PWM LQR Mode(straight) function     *
* void pwm_lqr_rotate(int,int);      PWM LQR Mode(rotate) function       *
* void pwm_lqr_circle(int,int,int);  PWM LQR Mode(post mode) function    *
* void data_write(int,int,int,int);  PWM Data Wirte in FIFO function     *
*************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "pwm_enc.h"

int data_write(int, int, int, int);

void pwm_both_data(int l_duty, int r_duty){  /* [duty] [duty] */
	int left_vdata, right_vdata; // [duty] [duty]

	left_vdata = (l_duty >= 0) ? l_duty : (-l_duty+128);
	right_vdata = (r_duty >= 0) ? r_duty : (-r_duty+128);
	data_write(left_vdata, right_vdata, 0, DUTY_MODE);
}

void pwm_straight(int l_distance, int r_distance){ /* [mm] [mm] */
	int l_speed, r_speed; // [target_count] [target_count]

	l_speed = (int)(((l_distance) / (diameter*pi)) * gear * cpr * adj);
	r_speed = (int)(((r_distance) / (diameter*pi)) * gear * cpr * adj);
	data_write(l_speed, r_speed, 2, PID_MODE);
}

void pwm_rotate(int angle){ /* [deg] */
	int l_speed, r_speed; // [target_count] [target_count]

	l_speed = (int)((-1 * intarval * angle / diameter) * gear * cpr / 360 * 1.0);
	r_speed = (int)((intarval * angle / diameter) * gear * cpr / 360 * 1.0);
	data_write(l_speed, r_speed, 2, PID_MODE);
}

void pwm_lqr_straight(int distance, int time){ /* [mm] [msec] */
	int l_distance, r_distance; // [mm] [mm]
	
	l_distance = (int)distance;
	r_distance = (int)distance;
	data_write(l_distance, r_distance, abs(time), LQR_MODE);
}

void pwm_lqr_rotate(int angle, int time){ /* [deg] [msec] */
	int distance, l_distance, r_distance; // [mm] [mm] [mm]
	
	distance=(int)2 * dist_ct * pi * abs(angle) / 360;
	l_distance = (angle > 0) ? -distance : distance;
	r_distance = (angle > 0) ? distance : -distance;
	data_write(l_distance, r_distance, abs(time), LQR_MODE);
	
}

void pwm_lqr_circle(int radius, int angle, int time){ /* [mm] [deg] [msec] */
	int l_radius, r_radius, l_distance, r_distance; // [mm] [mm] [mm] [mm]
	
	l_radius = (radius > 0) ? radius-dist_ct : abs(radius)+dist_ct;
	r_radius = (radius > 0) ? radius+dist_ct : abs(radius)-dist_ct;
	l_distance = (int)2 * l_radius * pi * angle / 360;
	r_distance = (int)2 * r_radius * pi * angle / 360;
	data_write(l_distance, r_distance, abs(time), LQR_MODE);
}

// Data Wirte function
int data_write(int left, int right, int Tup, int mode){
	int fd2;
	struct pwm_speed_data pwm;
	// FIFO Open
	if((fd2 = open("/dev/rtf4", O_WRONLY)) < 0){
		fprintf(stderr, "Error opening /dev/rtf4\n");
		return 0;
	}
	// Input pwm data
	pwm.l_speed = left;
	pwm.r_speed = right;
	pwm.tup = Tup;
	pwm.mode = mode;
	/*printf("PWM_L=%d,PWM_R=%d\n",left,right);*/
	// Write pwm data to FIFO
	if (write(fd2, &pwm, sizeof(pwm)) < 0){
		fprintf(stderr, "Can't send a data to RT-task\n");
	}
	close(fd2);
	return 0;
}
