#include <linux/errno.h>
#include <rtl.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_fifo.h>
#include <asm/io.h>
#include <rtl_core.h>
#include <rtl_time.h>
#include <linux/ioport.h>

#include "uss.h"

pthread_t tasks0;
struct uss_msg_struct msg;

hrtime_t r_time;
hrtime_t s_time;
int i;
int data;
int iflag;

unsigned int transmit(int i)
{
	iflag=0;
	outb(1, 0x132);
	s_time = gethrtime();
	outb(i, 0x130);
	rtl_hard_enable_irq (USS_IRQ); 
        return 0;
}

int command_handler(unsigned int cmd)
{
	int err;
 	while ((err = rtf_get(FIFO_1, &msg, sizeof(msg))) == sizeof(msg)) {

		if(msg.command==USS_SHOT){
			transmit(msg.number-1);
		}

		if(msg.command==USS_START_TASK){
                         pthread_make_periodic_np(tasks0, gethrtime(), (long long)msg.period * (long long)1000000);
                         pthread_wakeup_np(tasks0);
		}
		if(msg.command==USS_STOP_TASK){
			 pthread_suspend_np(tasks0);
		}

	}

	return 0;
}

void *thread_uss(void *t)
{
	iflag=1;
        while(1){
		if(iflag==0){
			data=9999;
        		rtf_put(FIFO_2, &data, sizeof(data));
		}
		pthread_wait_np();
		transmit(msg.number-1);
//                intr_on=0;
//                      rtl_printf("uss transmitted\n");
//                      rtl_printf("uss transmitted No=%d\n",msg.number);
        }
        return 0;
}


unsigned int intr_handler(unsigned int irq, struct pt_regs *regs)
{
	outb(2, 0x132);
	r_time = gethrtime();
        data = ((int)((float)r_time - (float)s_time)) * 343.7 / (1000000 * 2);
	iflag=1;
        rtf_put(FIFO_2, &data, sizeof(data));
        //rtl_printf("data =  %d mm.\n passed intr_handler\n", data);
	outb(1, 0x132);
	rtl_hard_disable_irq (USS_IRQ);
        return 0;
}

int init_module(void)
{	
        pthread_attr_t attr;
        struct sched_param sched_param;

 	rtf_create(1, 4000);
	rtf_create(2, 4000);

	if( check_region(0x130,6) != 0){
		printk("I/O error.\n");
 		return -ENODEV;
	}

  	request_region(0x130,6,"USS");

        rtf_create_handler(FIFO_1, &command_handler);
 	if(rtl_request_irq (USS_IRQ, intr_handler) != 0) {
		release_region(0x130,6);
		printk("IRQ error.\n");
		return -ENODEV;
	}

        pthread_attr_init(&attr);
        sched_param.sched_priority = 5;
        pthread_attr_setschedparam(&attr, &sched_param);
        pthread_create(&tasks0, &attr, &thread_uss, (void *)0);

	return 0;
}

void cleanup_module(void)
{
	rtf_destroy(1);
	rtf_destroy(2);
	release_region(0x130,6);
	pthread_delete_np (tasks0);
	rtl_free_irq (USS_IRQ); 
}
